package container;


import lombok.extern.slf4j.Slf4j;

import java.math.BigDecimal;
import java.util.Map;

/**
 * 协议采集到数据的容器
 *
 * @author 修唯xiuwei
 * @version 3.0
 */

@Slf4j
public abstract class ProtocolDataContainer {


    private static final ProtocolDataContainer INSTANCE;

    static {
            INSTANCE=ProtocolDataContainer_Local.getInstance();
    }

    protected ProtocolDataContainer(){

    }



    /**
     * 获取单例实例
     *
     * @return the instance
     */
    public static ProtocolDataContainer getInstance() {
        return INSTANCE;
    }

    /**
     * 向池子内置数据
     *
     * @param point point
     * @param value value
     */
    public abstract void putNumber(String point, BigDecimal value);


    /**
     * 向池子内置采集数据
     *
     * @param point point
     * @param value value
     */
    public void putGNumber(Integer point, BigDecimal value) {
        this.putNumber("G" + point, value);
    }


    /**
     * 向池子内置配置数据
     *
     * @param point point
     * @param value value
     */
    public void putCNumber(Integer point, BigDecimal value) {
        this.putNumber("C" + point, value);
    }

    /**
     * 向池子内置计算得出数据
     *
     * @param point point
     * @param value value
     */
    public void putJNumber(Integer point, BigDecimal value) {
        this.putNumber("J" + point, value);
    }


    /**
     * 向池子内置数据
     *
     * @param point point
     * @param value value
     */
    public void putGBoolean(Integer point, Boolean value) {
        this.putBoolean("G" + point, value);
    }


    /**
     * 向池子内置数据
     *
     * @param point point
     * @param value value
     */
    public abstract void putBoolean(String point, Boolean value);

    /**
     * 向池子内置数据
     *
     * @param point point
     * @param value value
     */
    public void putCBoolean(Integer point, Boolean value) {
        this.putBoolean("C" + point, value);
    }

    /**
     * 向池子内置数据
     *
     * @param point point
     * @param value value
     */
    public void putJBoolean(Integer point, Boolean value) {
        this.putBoolean("J" + point, value);
    }


    /**
     * 获取到缓存池内的数值 如果遥测池内没有但是存在于遥信池 将返回 0or1
     *
     * @param point 点位
     * @return Number number
     */
    public abstract BigDecimal getNumber(String point);

    /**
     * 获取到返回池内的遥信数据  如果遥信池内没有 遥测池内有 那么大于0的数值 将会返回ture 否则为false
     *
     * @param point 点位
     * @return Boolean boolean
     */
    public abstract Boolean getBoolean(String point);

    /**
     * 遥信数据池是否含有该点位
     *
     * @param point point
     * @return the boolean
     */
    public abstract Boolean isYxContains(String point);

    /**
     * 遥测数据池是否含有该点位值
     *
     * @param point point
     * @return the boolean
     */
    public abstract Boolean isYcContains(String point);


    public abstract Map<String, Boolean> getYxMap();

    public abstract Map<String, BigDecimal> getYcMap();




}
