package tunnelworker.masters.iml;


import container.ProtocolTunnelContainer;
import entity.datapoint.CommandDataPoint;
import entity.datapoint.GatherDataPoint;
import enums.TunnelStatus;
import exception.DataExchangeException;
import lombok.Getter;
import mastertunnelinfo.MasterCdtRtuTunnelInfo;
import tunnelworker.BaseProtocolTunnel;
import tunnelworker.masters.MasterInterface;
import tunnelworker.workassist.SingleThreadPoolExecutorUtil;
import tunnelworker.workassist.cdt.MyCDTDataHandler;
import wei.yigulu.cdt.netty.CDTMaster;
import wei.yigulu.modbus.domain.datatype.ModbusDataTypeEnum;
import wei.yigulu.purejavacomm.PureJavaCommChannelConfig;

import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * cdt 协议 Rtu获取
 *
 * @author: xiuwei
 * @version:
 */
@SuppressWarnings("ALL")
public class CdtRtuMaster extends BaseProtocolTunnel<MasterCdtRtuTunnelInfo, CDTMaster> implements MasterInterface {


    /**
     * 遥测点表
     */
    @Getter
    private Map<Integer, GatherDataPoint> cdtYcDataPoints;
    /**
     * 遥信点表
     */
    @Getter
    private Map<Integer, GatherDataPoint> cdtYxDataPoints;


    /**
     * 构造方法
     *
     * @param gatherCdtRtuTunnelInfo 通道信息
     */
    public CdtRtuMaster(MasterCdtRtuTunnelInfo gatherCdtRtuTunnelInfo) {
        super(gatherCdtRtuTunnelInfo);
    }


    @Override
    protected CdtRtuMaster buildTunnel() throws DataExchangeException {
        parseDataPoint();
        this.protocolBuilder = new CDTMaster(getTunnelInfo().getSerialName(), new MyCDTDataHandler(this).setLog(log));
        this.protocolBuilder.setLog(log);
        this.protocolBuilder.setBaudRate(getTunnelInfo().getBaudRate()).setParity(PureJavaCommChannelConfig.Paritybit.valueOf(getTunnelInfo().getParity()));
        this.setTunnelStatus(TunnelStatus.BUILT);
        ProtocolTunnelContainer.getInstance().addTunnel(this);
        log.info("成功创建CDT  rtu Master通道对象：{}", this.tunnelInfo.getTunnelName());
        return this;
    }

    @Override
    protected void parseDataPoint() throws DataExchangeException {
        List<GatherDataPoint> dataPoints = this.getTunnelInfo().getDataPoints();
        cdtYxDataPoints = new HashMap<>();
        cdtYcDataPoints = new HashMap<>();
        if (dataPoints == null || dataPoints.isEmpty()) {
            return;
        }
        for (GatherDataPoint d : dataPoints) {
            if (ModbusDataTypeEnum.A16.equals(d.getDataType().getModbusDataType())) {
                cdtYxDataPoints.put(d.getProtocolPoint(), d);
            } else {
                cdtYcDataPoints.put(d.getProtocolPoint(), d);
            }
        }

    }


    @Override
    public BaseProtocolTunnel tunnelStop() throws DataExchangeException {
        if (this.protocolBuilder != null) {
            protocolBuilder.stop();
        }
        log.info("关闭 cdt Master 通道 {}", this.tunnelInfo.getTunnelName());
        return super.tunnelStop();
    }


    @Override
    public CdtRtuMaster startTunnel() throws DataExchangeException {
        log.info("CDT RTU master {} 通道开始连接", getTunnelInfo().getTunnelName());
        protocolTunnelContainer.addUpdateDateTask(this);
        try {
            SingleThreadPoolExecutorUtil.executeBySingleThreadExecutor(() -> {
                this.protocolBuilder.create();
                log.error("cdt rtu master  创建通道失败");
                setTunnelStatus(TunnelStatus.LISTENSERIALFAIL);
            });
        } catch (Exception e) {
            log.error("cdt rtu master 创建通道失败", e);
            setTunnelStatus(TunnelStatus.LISTENSERIALFAIL);
            throw new DataExchangeException(10007, "cdt rtu master创建通道失败");
        }
        if (!TunnelStatus.LISTENSERIALFAIL.equals(getTunnelStatus())) {
            setTunnelStatus(TunnelStatus.LISTENSERIALSUCCESS);
        }
        return this;
    }

    @Override
    public void getData() throws DataExchangeException {
        return;
    }

    @Override
    public boolean sendCommand(CommandDataPoint dataPoint, Object val) throws DataExchangeException {
        return false;
    }



}
