package tunnelworker.masters.iml;


import enums.TunnelStatus;
import exception.DataExchangeException;
import mastertunnelinfo.MasterModbusTcpTunnelInfo;
import tunnelworker.BaseProtocolTunnel;
import tunnelworker.workassist.SingleThreadPoolExecutorUtil;
import wei.yigulu.modbus.netty.HSModbusTcpMasterBuilder;

/**
 * modbus协议TCP数据采集
 *
 * @author 修唯xiuwei
 * @version 3.0
 */

public class ModbusTcpMaster extends BaseModbusMaster<MasterModbusTcpTunnelInfo, HSModbusTcpMasterBuilder> {


    /**
     * 构造方法
     *
     * @param gatherModbusTcpTunnelInfo 通道信息
     */
    public ModbusTcpMaster(final MasterModbusTcpTunnelInfo gatherModbusTcpTunnelInfo) {
        super(gatherModbusTcpTunnelInfo);
    }

    @Override
    protected ModbusTcpMaster buildTunnel() throws DataExchangeException {
        setSlaveId(tunnelInfo.getSlaveId());
        protocolBuilder = new ModbusMaster(tunnelInfo.getRemoteIp(), tunnelInfo.getRemotePort());
        protocolBuilder.setSpareIp(tunnelInfo.getRemoteSpareIp());
        protocolBuilder.setSparePort(tunnelInfo.getRemoteSparePort());
        protocolBuilder.setSelfIp(tunnelInfo.getSelfIp());
        protocolBuilder.setSelfPort(tunnelInfo.getSelfPort());
        protocolBuilder.setLog(this.log);
        setTunnelStatus(TunnelStatus.BUILT);
        protocolTunnelContainer.addTunnel(this);
        log.info("成功创建ModbusTCPMaster通道对象：{}", this.tunnelInfo.getTunnelName());
        return this;
    }


    @Override
    public ModbusTcpMaster startTunnel() throws DataExchangeException {
        log.info("modbus TCP master {} 通道开始连接", getTunnelInfo().getTunnelName());
        protocolTunnelContainer.addUpdateDateTask(this);
        try {
            SingleThreadPoolExecutorUtil.executeBySingleThreadExecutor(() -> {
                try {
                    this.protocolBuilder.create();
                    log.error("modbus TCP master  创建通道失败");
                    setTunnelStatus(TunnelStatus.LISTENPORTFAIL);
                } catch (Exception e) {
                    log.error("modbus TCP master  创建通道失败", e);
                    setTunnelStatus(TunnelStatus.LISTENPORTFAIL);
                }
            });
        } catch (Exception e) {
            log.error("modbus TCP master 创建通道失败", e);
            setTunnelStatus(TunnelStatus.LISTENPORTFAIL);
            throw new DataExchangeException(10007, "modbus TCP master创建通道失败");
        }
        if (!TunnelStatus.LISTENPORTFAIL.equals(getTunnelStatus())) {
            setTunnelStatus(TunnelStatus.LISTENPORTSUCCESS);
        }
        return this;
    }


    @Override
    public void getData() throws DataExchangeException {
        try {
            super.getData();
        } catch (DataExchangeException e) {
            this.setTunnelStatus(TunnelStatus.CONNECTEDANDCOMMERROR);
            throw e;
        }
        this.setTunnelStatus(TunnelStatus.CONNECTED);
    }


    @Override
    public BaseProtocolTunnel tunnelStop() throws DataExchangeException {
        if (this.protocolBuilder != null) {
            this.protocolBuilder.stop();
        }
        log.info("关闭 modbus TCP master 通道 {}", this.tunnelInfo.getTunnelName());
        return super.tunnelStop();
    }


    class ModbusMaster extends HSModbusTcpMasterBuilder {

        public ModbusMaster(String ip, Integer port) {
            super(ip, port);
        }

        @Override
        public void connected() {
            setTunnelStatus(TunnelStatus.CONNECTED);
        }

    }

}
