package tunnelworker.slavers.iml;


import container.ProtocolDataContainer;
import entity.datapoint.SenderDataPoint;
import enums.TunnelStatus;
import exception.DataExchangeException;
import slavertunnelinfo.SlaverModbusRtuTunnelInfo;
import tunnelworker.BaseProtocolTunnel;
import tunnelworker.slavers.SlaverInterface;
import tunnelworker.workassist.SingleThreadPoolExecutorUtil;
import wei.yigulu.modbus.domain.datatype.BooleanModbusDataInRegister;
import wei.yigulu.modbus.domain.datatype.ModbusDataTypeEnum;
import wei.yigulu.modbus.domain.datatype.NumericModbusData;
import wei.yigulu.modbus.netty.ModbusRtuSlaverBuilder;
import wei.yigulu.purejavacomm.PureJavaCommChannelConfig;

import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * modbus Rtu 的数据转发通道
 *
 * @author: xiuwei
 * @version:
 */

public class ModbusRtuSlaver extends BaseProtocolTunnel<SlaverModbusRtuTunnelInfo, ModbusRtuSlaverBuilder> implements SlaverInterface {


    /**
     * 通道点位  ---- 点位对象
     */
    Map<Integer, SenderDataPoint> dataPointMap;


    /**
     * 0: 线圈   1：寄存器
     */
    private int functionFlag;


    /**
     * 顶层的构造方法
     *
     * @param tunnelInfo 通道信息
     */
    public ModbusRtuSlaver(SlaverModbusRtuTunnelInfo tunnelInfo) {
        super(tunnelInfo);
    }

    @Override
    protected ModbusRtuSlaver buildTunnel() throws DataExchangeException {
        protocolBuilder = new ModbusRtuSlaverBuilder(this.tunnelInfo.getSerialName());
        protocolBuilder.setBaudRate(tunnelInfo.getBaudRate());
        protocolBuilder.setDataBits(PureJavaCommChannelConfig.Databits.valueOf(tunnelInfo.getDataBits()));
        protocolBuilder.setStopBits(PureJavaCommChannelConfig.Stopbits.valueOf(tunnelInfo.getStopBits()));
        protocolBuilder.setParity(PureJavaCommChannelConfig.Paritybit.valueOf(tunnelInfo.getParity()));
        protocolBuilder.setLog(this.log);
        log.info("成功创建ModbusRTUSlaver通道对象：{}", this.tunnelInfo.getTunnelName());
        setTunnelStatus(TunnelStatus.BUILT);
        protocolTunnelContainer.addTunnel(this);
        return this;
    }

    @Override
    protected void parseDataPoint() throws DataExchangeException {
        log.info("解析该通道下所管理的点位");
        List<SenderDataPoint> dataPoints = this.tunnelInfo.getDataPoints();
        if (dataPoints == null || dataPoints.isEmpty()) {
            return;
        }
        this.dataPointMap = new HashMap<>();
        for (SenderDataPoint d : dataPoints) {
            this.dataPointMap.put(d.getProtocolPoint(), d);
        }
        log.info("解析该通道下所管理的点位完成");
    }


    @Override
    public ModbusRtuSlaver startTunnel() throws DataExchangeException {
        log.info("modbus RTU slaver {} 通道开始连接", getTunnelInfo().getTunnelName());
        protocolTunnelContainer.addUpdateDateTask(this);
        try {
            SingleThreadPoolExecutorUtil.executeBySingleThreadExecutor(() -> {
                try {
                    this.protocolBuilder.create();
                    log.error("Modbus Rtu slaver  创建通道失败");
                    setTunnelStatus(TunnelStatus.LISTENSERIALFAIL);
                } catch (Exception e) {
                    log.error("Modbus Rtu slaver  创建通道失败", e);
                    setTunnelStatus(TunnelStatus.LISTENSERIALFAIL);
                }
            });
        } catch (Exception e) {
            log.error("Modbus Rtu slaver 创建通道失败", e);
            setTunnelStatus(TunnelStatus.LISTENSERIALFAIL);
            throw new DataExchangeException(10007, "Modbus Rtu slaver创建通道失败");
        }
        if (!TunnelStatus.LISTENSERIALFAIL.equals(getTunnelStatus())) {
            setTunnelStatus(TunnelStatus.LISTENSERIALSUCCESS);
        }
        return this;
    }


    @Override
    public BaseProtocolTunnel tunnelStop() throws DataExchangeException {
        if (this.protocolBuilder != null) {
            this.protocolBuilder.stop();
        }
        log.info("关闭 Modbus Rtu slaver 通道 {}", this.tunnelInfo.getTunnelName());
        return super.tunnelStop();
    }


    @Override
    public void updateData2Protocol() throws DataExchangeException {
        ProtocolDataContainer dataContainer = ProtocolDataContainer.getInstance();
        if (functionFlag == 1) {
            this.dataPointMap.forEach((k, v) -> {
                if (ModbusDataTypeEnum.A16.equals(v.getDataType().getModbusDataType())) {
                    BooleanModbusDataInRegister booleanModbusDataInRegister = new BooleanModbusDataInRegister();
                    booleanModbusDataInRegister.setValue(0, dataContainer.getBoolean(v.getDataPointId()));
                    this.protocolBuilder.getModbusSlaveDataContainer().setRegister(this.tunnelInfo.getSlaveId(), k, booleanModbusDataInRegister);
                } else {
                    this.protocolBuilder.getModbusSlaveDataContainer().setRegister(this.tunnelInfo.getSlaveId(), k, ((NumericModbusData) (v.getDataType().getModbusDataType().getObject())).setValue(dataContainer.getNumber(v.getDataPointId()).multiply(v.getRatio())));
                }
            });
        } else {
            this.dataPointMap.forEach((k, v) ->
                    this.protocolBuilder.getModbusSlaveDataContainer().setCoil(this.tunnelInfo.getSlaveId(), k, dataContainer.getBoolean(v.getDataPointId()))
            );
        }
    }
}
