package tunnelworker.workassist;


import entity.BaseTunnelInfo;
import exception.DataExchangeException;
import mastertunnelinfo.Master104TcpTunnelInfo;
import mastertunnelinfo.MasterCdtRtuTunnelInfo;
import mastertunnelinfo.MasterModbusRtuTunnelInfo;
import mastertunnelinfo.MasterModbusTcpTunnelInfo;
import slavertunnelinfo.Slaver104TcpTunnelInfo;
import slavertunnelinfo.SlaverCdtRtuTunnelInfo;
import slavertunnelinfo.SlaverModbusRtuTunnelInfo;
import slavertunnelinfo.SlaverModbusTcpTunnelInfo;
import tunnelworker.BaseProtocolTunnel;
import tunnelworker.masters.iml.CdtRtuMaster;
import tunnelworker.masters.iml.Iec104TcpMaster;
import tunnelworker.masters.iml.ModbusRtuMaster;
import tunnelworker.masters.iml.ModbusTcpMaster;
import tunnelworker.slavers.iml.CdtRtuSlaver;
import tunnelworker.slavers.iml.Iec104TcpSlaver;
import tunnelworker.slavers.iml.ModbusRtuSlaver;
import tunnelworker.slavers.iml.ModbusTcpSlaver;

/**
 * 用Tunnel Info 创建通达
 *
 * @author: xiuwei
 * @version:
 */
public class TunnelBuilder {

	private TunnelBuilder() {
		throw new IllegalStateException("Utility class");
	}

	/**
	 * 建立通道    主要是加载通道的配置 信息  以及点位信息
	 *
	 * @param tunnelInfo 隧道信息
	 * @return {@link BaseProtocolTunnel}
	 * @throws DataExchangeException 数据交换异常
	 */
	public static BaseProtocolTunnel buildTunnel(BaseTunnelInfo tunnelInfo) throws DataExchangeException {
		switch (tunnelInfo.getTunnelType()) {
			case IEC104TCPMASTER:
				return new Iec104TcpMaster((Master104TcpTunnelInfo) tunnelInfo).build();
			case IEC104TCPSLAVE:
				return new Iec104TcpSlaver((Slaver104TcpTunnelInfo) tunnelInfo).build();
			case MODBUSRTUMASTER:
				return new ModbusRtuMaster((MasterModbusRtuTunnelInfo) tunnelInfo).build();
			case MODBUSTCPMASTER:
				return new ModbusTcpMaster((MasterModbusTcpTunnelInfo) tunnelInfo).build();
			case MODBUSRTUSLAVE:
				return new ModbusRtuSlaver((SlaverModbusRtuTunnelInfo) tunnelInfo).build();
			case MODBUSTCPSLAVE:
				return new ModbusTcpSlaver((SlaverModbusTcpTunnelInfo) tunnelInfo).build();
			case CDTRTUSLAVE:
				return new CdtRtuSlaver((SlaverCdtRtuTunnelInfo) tunnelInfo).build();
			case CDTRTUMASTER:
				return new CdtRtuMaster((MasterCdtRtuTunnelInfo) tunnelInfo).build();
			default:
				return null;
		}
	}


}


