package tunnelworker.workassist.cdt;


import container.ProtocolDataContainer;
import enums.TunnelStatus;
import lombok.NonNull;
import tunnelworker.masters.iml.CdtRtuMaster;
import wei.yigulu.cdt.cdtframe.AbstractCDTDataHandler;
import wei.yigulu.cdt.cdtframe.BaseDateType;

import java.math.BigDecimal;
import java.util.List;

/**
 * 基于数据模块的cdt的数据处理类
 *
 * @author: xiuwei
 * @version:
 */
public class MyCDTDataHandler extends AbstractCDTDataHandler {

    ProtocolDataContainer protocolDataContainerRedis = ProtocolDataContainer.getInstance();


    /**
     * cdt rtu 的数据采集对象 用于采集数据
     */
    private CdtRtuMaster cdtRtuDataGather;

    /**
     * 构造方法
     *
     * @param cdtRtuDataGather cdt 采集通道实体
     */
    public MyCDTDataHandler(@NonNull CdtRtuMaster cdtRtuDataGather) {
        this.cdtRtuDataGather = cdtRtuDataGather;
    }


    @Override
    protected void processYx(List<BaseDateType> dates) {
        for (BaseDateType dateType : dates) {
            dateType.getDates().forEach((k, v) -> {
                if (this.cdtRtuDataGather.getCdtYxDataPoints().containsKey(k)) {
                    protocolDataContainerRedis.putGBoolean(this.cdtRtuDataGather.getCdtYxDataPoints().get(k).getId(), (Boolean) v);
                }
            });
        }
    }

    /**
     * 不分重要 次要和普通 统一处理遥测
     *
     * @param dates
     */
    private void processYc(List<BaseDateType> dates) {
        for (BaseDateType dateType : dates) {
            getLog().info("接收到数据：{}", dateType.getDates());
            dateType.getDates().forEach((k, v) -> {
                if (this.cdtRtuDataGather.getCdtYcDataPoints().containsKey(k)) {
                    protocolDataContainerRedis.putGNumber(this.cdtRtuDataGather.getCdtYcDataPoints().get(k).getId(), BigDecimal.valueOf((Integer) v).multiply(this.cdtRtuDataGather.getCdtYcDataPoints().get(k).getRatio()));
                }
            });
        }
    }


    @Override
    protected void processImportantYc(List<BaseDateType> list) {
        processYc(list);
    }

    @Override
    protected void processSecondYc(List<BaseDateType> list) {
        processYc(list);
    }

    @Override
    protected void processCommonYc(List<BaseDateType> list) {
        processYc(list);
    }

    @Override
    public void connected() {
        this.cdtRtuDataGather.setTunnelStatus(TunnelStatus.LISTENSERIALSUCCESS);
    }


    @Override
    public void disconnected() {
        this.cdtRtuDataGather.setTunnelStatus(TunnelStatus.LISTENSERIALFAIL);
    }
}
